import React from "react";
import { ReactNode } from "react";
import { Node } from "../../common/graph/node";
import { NodeType } from "../../common/graph/nodetype";
import "./nodedetails.css";

type propTypes = {
    selectedNodes: Node[];
    allTypes: NodeType[];
    onChange: { (): void };
};

export class NodeDetails extends React.Component<propTypes> {
    debounceTimeout: NodeJS.Timeout;
    debounceArgs: any[];
    debounceFunc: any;

    constructor(props: propTypes) {
        super(props);
        this.handleNodeTypeChange = this.handleNodeTypeChange.bind(this);
        this.handleTextChange = this.handleTextChange.bind(this);
        this.debounce = this.debounce.bind(this);
    }

    private handleNodeTypeChange(event: any) {
        this.props.selectedNodes.forEach(
            (n: Node) => n.setType(event.target.value) // TODO: Later implement new save point handling to collect them all into a big one
        );
        this.props.onChange();
    }

    private get referenceNode(): Node {
        if (
            this.props.selectedNodes == undefined ||
            this.props.selectedNodes.length <= 0
        ) {
            // Nothing selected
            return new Node();
        } else if (this.props.selectedNodes.length === 1) {
            // Single node handling
            return this.props.selectedNodes[0];
        } else {
            // Multiple nodes selected => Create a kind of merged node
            const refNode = new Node();
            Object.assign(refNode, this.props.selectedNodes[0]);

            refNode.banner = this.getCollectiveValue((n: Node) => n.banner);
            refNode.icon = this.getCollectiveValue((n: Node) => n.icon);
            refNode.video = this.getCollectiveValue((n: Node) => n.video);
            refNode.type = this.getCollectiveValue((n: Node) => n.type);

            return refNode;
        }
    }

    /**
     * Tries to find a representative value for a specific property over all selected nodes.
     * @param propGetter Function that returns the value to test for each node.
     * @returns If all nodes have the same value, this value is returned. Otherwise undefined is returned.
     */
    getCollectiveValue(propGetter: (n: Node) => any): any {
        const sameValue: any = propGetter(this.props.selectedNodes[0]);

        const differentValueFound = this.props.selectedNodes.some(
            (n: Node) => propGetter(n) !== sameValue
        );
        if (differentValueFound) {
            return undefined;
        }

        return sameValue;
    }

    /**
     * Generic function for handeling a changing text input and applying the new value to the currently selected node.
     * @param event Change event of text input.
     * @param property Property to give new value.
     */
    private handleTextChange(event: any, property: string) {
        const newValue = event.target.value;

        // Actual change?
        if ((this.referenceNode as any)[property] == newValue) {
            return;
        }

        this.props.selectedNodes.forEach((n: any) => (n[property] = newValue));
        this.props.onChange();

        // Save change, but debounce, so it doesn't trigger too quickly
        this.debounce(
            (property: string) => {
                this.props.selectedNodes[0].graph.storeCurrentData(
                    "Changed " + property + " of selected nodes"
                );
                this.props.onChange();
            },
            500,
            property
        );
    }

    debounce(func: any, wait: number, ...args: any[]) {
        // It works, don't question it
        const later = () => {
            this.debounceTimeout = null;
            this.debounceFunc(...this.debounceArgs);
        };

        clearTimeout(this.debounceTimeout);
        if (
            this.debounceArgs !== undefined &&
            args[0] !== this.debounceArgs[0] &&
            this.debounceFunc !== undefined
        ) {
            this.debounceFunc(...this.debounceArgs);
        }

        this.debounceArgs = args;
        this.debounceFunc = func;
        this.debounceTimeout = setTimeout(later, wait);
    }

    render(): ReactNode {
        if (
            this.props.selectedNodes === undefined ||
            this.props.selectedNodes.length <= 0
        ) {
            return <p>No Node selected.</p>;
        }

        return (
            <div id="nodedetails">
                {this.props.selectedNodes.length === 1 ? (
                    <div>
                        <label htmlFor="node-name" hidden>
                            Name
                        </label>
                        <input
                            type="text"
                            id="node-name"
                            name="node-name"
                            placeholder="Enter name"
                            className="bottom-space"
                            value={this.referenceNode.name}
                            onChange={(event) =>
                                this.handleTextChange(event, "name")
                            }
                        ></input>
                    </div>
                ) : (
                    <h3>{this.props.selectedNodes.length} nodes selected</h3>
                )}

                {this.props.selectedNodes.length === 1 ? (
                    <div>
                        <label htmlFor="node-description">Description</label>
                        <br />
                        <textarea
                            id="node-description"
                            name="node-description"
                            className="bottom-space"
                            value={this.referenceNode.description ?? ""}
                            onChange={(event) =>
                                this.handleTextChange(event, "description")
                            }
                        ></textarea>
                    </div>
                ) : (
                    ""
                )}
                <div>
                    <label htmlFor="node-image">Icon Image</label>
                    <br />
                    {this.referenceNode.icon ? (
                        <div>
                            <img
                                id="node-image-preview"
                                className="preview-image"
                                src={this.referenceNode.icon}
                            />
                            <br />
                        </div>
                    ) : (
                        ""
                    )}
                    <input
                        type="text"
                        id="node-image"
                        name="node-image"
                        placeholder="Image URL"
                        className="bottom-space"
                        value={this.referenceNode.icon ?? ""}
                        onChange={(event) =>
                            this.handleTextChange(event, "icon")
                        }
                    />
                </div>
                <div>
                    <label htmlFor="node-detail-image">Banner Image</label>
                    <br />
                    {this.referenceNode.banner ? (
                        <div>
                            <img
                                id="node-image-preview"
                                className="preview-image"
                                src={this.referenceNode.banner}
                            />
                            <br />
                        </div>
                    ) : (
                        ""
                    )}
                    <input
                        type="text"
                        id="node-detail-image"
                        name="node-detail-image"
                        placeholder="Image URL"
                        className="bottom-space"
                        value={this.referenceNode.banner ?? ""}
                        onChange={(event) =>
                            this.handleTextChange(event, "banner")
                        }
                    />
                </div>
                <div>
                    <label htmlFor="node-type">Type</label>
                    <br />
                    <select
                        id="node-type"
                        name="node-type"
                        className="bottom-space"
                        value={
                            this.referenceNode.type
                                ? this.referenceNode.type.id
                                : ""
                        }
                        onChange={this.handleNodeTypeChange}
                    >
                        <option
                            className="empty-select-option"
                            disabled
                            selected
                        ></option>
                        {this.props.allTypes.map((type) => (
                            <option key={type.id} value={type.id}>
                                {type.name}
                            </option>
                        ))}
                    </select>
                </div>
                <div>
                    <label htmlFor="node-video">Video</label>
                    <br />
                    <input
                        type="text"
                        placeholder="Video URL"
                        id="node-video"
                        name="node-video"
                        value={this.referenceNode.video ?? ""}
                        onChange={(event) =>
                            this.handleTextChange(event, "video")
                        }
                    ></input>
                </div>
                {this.props.selectedNodes.length === 1 ? (
                    <div>
                        <label htmlFor="node-references">References</label>{" "}
                        <small>One URL per line</small>
                        <br />
                        <textarea
                            id="node-references"
                            name="node-references"
                            className="bottom-space"
                            value={this.referenceNode.references}
                            onChange={(event) =>
                                this.handleTextChange(event, "references")
                            }
                        ></textarea>
                    </div>
                ) : (
                    ""
                )}
            </div>
        );
    }
}
